@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,31 +33,30 @@
 #ifndef NBDKIT_BLK_H
 #define NBDKIT_BLK_H
 
-/* Size of a block in the overlay.  A 4K block size means that we need
- * 32 MB of memory to store the bitmap for a 1 TB underlying image.
- */
-#define BLKSIZE 4096
-
 /* Initialize the overlay and bitmap. */
 extern int blk_init (void);
 
 /* Close the overlay, free the bitmap. */
 extern void blk_free (void);
 
-/*----------------------------------------------------------------------
- * ** NOTE **
- *
- * An exclusive lock must be held when you call any function below
- * this line.
- */
-
 /* Allocate or resize the overlay and bitmap. */
 extern int blk_set_size (uint64_t new_size);
 
+/* Returns the status of the block in the overlay. */
+extern void blk_status (uint64_t blknum, bool *present, bool *trimmed);
+
 /* Read a single block from the overlay or plugin. */
-extern int blk_read (struct nbdkit_next_ops *next_ops, void *nxdata,
-                     uint64_t blknum, uint8_t *block, int *err)
-  __attribute__((__nonnull__ (1, 4, 5)));
+extern int blk_read (nbdkit_next *next,
+                     uint64_t blknum, uint8_t *block,
+                     bool cow_on_read, int *err)
+  __attribute__ ((__nonnull__ (1, 3, 5)));
+
+/* Read multiple blocks from the overlay or plugin. */
+extern int blk_read_multiple (nbdkit_next *next,
+                              uint64_t blknum, uint64_t nrblocks,
+                              uint8_t *block,
+                              bool cow_on_read, int *err)
+  __attribute__ ((__nonnull__ (1, 4, 6)));
 
 /* Cache mode for blocks not already in overlay */
 enum cache_mode {
@@ -68,16 +67,17 @@ enum cache_mode {
 };
 
 /* Cache a single block from the plugin. */
-extern int blk_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+extern int blk_cache (nbdkit_next *next,
                       uint64_t blknum, uint8_t *block, enum cache_mode,
                       int *err)
-  __attribute__((__nonnull__ (1, 4, 6)));
+  __attribute__ ((__nonnull__ (1, 3, 5)));
 
 /* Write a single block. */
 extern int blk_write (uint64_t blknum, const uint8_t *block, int *err)
-  __attribute__((__nonnull__ (2, 3)));
+  __attribute__ ((__nonnull__ (2, 3)));
 
-/* Flush the overlay to disk. */
-extern int blk_flush (void);
+/* Trim a single block. */
+extern int blk_trim (uint64_t blknum, int *err)
+  __attribute__ ((__nonnull__ (2)));
 
 #endif /* NBDKIT_BLK_H */
