@@ -34,13 +34,13 @@ import base64
 import errno
 import os
 import re
+import secrets
 import tempfile
 import threading
 from typing import Any, Dict, Optional, Union, List
 import unittest
 from contextlib import closing, contextmanager
 from io import BytesIO
-import builtins
 from unittest.mock import patch
 
 import boto3
@@ -69,11 +69,11 @@ class Config:
         """Set a configuration value."""
 
         if key == "access-key" or key == "access_key":
-            self.access_key = value
+            self.access_key = nbdkit.read_password(value).decode("ascii")
         elif key == "secret-key" or key == "secret_key":
-            self.secret_key = value
+            self.secret_key = nbdkit.read_password(value).decode("ascii")
         elif key == "session-token" or key == "session_token":
-            self.session_token = value
+            self.session_token = nbdkit.read_password(value).decode("ascii")
         elif key == "endpoint-url" or key == "endpoint_url":
             self.endpoint_url = value
         elif key == "bucket":
@@ -140,24 +140,18 @@ def block_size(server):
         return (1, 512*1024, 0xffffffff)
 
     # Should return (minimum, preferred, maximum) block size. We return the
-    # same value for all three because even though the plugin can handle
-    # arbitrary large and small blocks, the performance penalty is huge and it
-    # is always preferable for the client to split up requests as needed.
-    return (cfg.obj_size, cfg.obj_size, cfg.obj_size)
-
-
-def is_rotational(server):
-    return False
+    # same value for preferred and maximum because even though the plugin can
+    # handle arbitrary large and small blocks, the performance penalty is
+    # huge, and it is always preferable for the client to split up requests
+    # as needed. At the same time, the minimum accessible block size is still
+    # 1 byte.
+    return (1, cfg.obj_size, cfg.obj_size)
 
 
 def can_fua(server):
     return nbdkit.FUA_NATIVE
 
 
-def can_extents(server):
-    return False
-
-
 def can_flush(server):
     return True
 
@@ -349,7 +343,7 @@ class Server:
                 off += cfg.obj_size
 
     def _put_object(self, obj_name: str, buf: BinaryData) -> None:
-        "Write *buf* into *obj_name"
+        """Write *buf* into *obj_name"""
 
         assert len(buf) == cfg.obj_size
 
@@ -363,9 +357,6 @@ class Server:
         if size == 0:
             return
 
-        if flags & nbdkit.FLAG_MAY_TRIM:
-            return self.trim(size, offset, 0)
-
         # Calculate block number and offset within the block for the
         # first byte that we need to write.
         (blockno1, block_offset1) = divmod(offset, cfg.obj_size)
@@ -400,17 +391,19 @@ class Server:
         if size == 0:
             return
 
+        # Note, NBD protocol allows to round the offset up and size down to
+        # meet internal alignment constraints.
+
         # Calculate block number and offset within the block for the
         # first byte that we need to trim.
         (blockno1, block_offset1) = divmod(offset, cfg.obj_size)
-
-        # Calculate block number of the last block that we have to
-        # trim fully.
-        (blockno2, _) = divmod(offset + size, cfg.obj_size)
-
         if block_offset1 != 0:
             blockno1 += 1
 
+        # Calculate block number of the block following the last one that we
+        # have to trim fully.
+        (blockno2, _) = divmod(offset + size, cfg.obj_size)
+
         if blockno1 == blockno2:
             nbdkit.debug('nothing to delete')
             return
@@ -456,7 +449,7 @@ class Server:
 
     def _list_objects(self, prefix: str,
                       start_after: Optional[str] = None) -> List[str]:
-        """Return keys for objects in  bucket.
+        """Return keys for objects in the bucket.
 
         Lists all keys starting with *prefix* in lexicographic order, starting
         with the key following *start_after*.
@@ -493,7 +486,7 @@ def concat(b1, b2):
 
 
 def put_object(s3, obj_name, buf):
-    "Write *buf* into *obj_name"
+    """Write *buf* into *obj_name"""
 
     assert len(buf) == cfg.obj_size
 
@@ -512,7 +505,7 @@ def make_client_error(errcode):
 class MultiLock:
     """Provides locking for large amounts of entities.
 
-    This class provides locking for a dynamically changing  and potentially
+    This class provides locking for a dynamically changing and potentially
     large set of entities, avoiding the need to allocate a separate lock for
     each entity. The `acquire` and `release` methods have an additional
     argument, the locking key, and only locks with the same key can see each
@@ -533,7 +526,7 @@ class MultiLock:
             self.release(key)
 
     def acquire(self, key):
-        '''Acquire lock for given key.'''
+        """Acquire lock for given key."""
 
         with self.cond:
             while key in self.locked_keys:
@@ -613,7 +606,6 @@ class LocalTest(unittest.TestCase):
         self.dev_size = 20*self.obj_size
         self.ref_fh = tempfile.TemporaryFile()
         self.ref_fh.truncate(cfg.dev_size)
-        self.rnd_fh = builtins.open('/dev/urandom', 'rb')
 
         config('bucket', 'testbucket')
         config('key', 'nbdkit_test')
@@ -628,10 +620,10 @@ class LocalTest(unittest.TestCase):
     def tearDown(self) -> None:
         super().tearDown()
         self.ref_fh.close()
-        self.rnd_fh.close()
 
-    def get_data(self, len):
-        buf = self.rnd_fh.read(len // 2 + 1)
+    @staticmethod
+    def get_data(len):
+        buf = secrets.token_bytes(len // 2 + 1)
         return base64.b16encode(buf)[:len]
 
     def compare_to_ref(self):
@@ -639,11 +631,11 @@ class LocalTest(unittest.TestCase):
         bl = self.obj_size
         buf = bytearray(bl)
         fh.seek(0)
-        for off in range(0, self.dev_size, self.obj_size):
+        for off in range(0, self.dev_size, bl):
             ref = fh.read(bl)
             pread(self.s3, buf, off, flags=0)
             self.assertEqual(
-                ref, buf, f'mismatch at off={off} (blk {off//self.obj_size})')
+                ref, buf, f'mismatch at off={off} (blk {off // bl})')
 
     def test_write_memoryview(self):
         buf = memoryview(bytearray(cfg.obj_size))
@@ -682,7 +674,7 @@ class LocalTest(unittest.TestCase):
 
         # Fill disk
         fh.seek(0)
-        for off in range(0, self.dev_size, self.obj_size):
+        for off in range(0, self.dev_size, bl):
             buf = self.get_data(bl)
             pwrite(self.s3, buf, offset=off, flags=0)
             fh.write(buf)
@@ -708,7 +700,7 @@ class LocalTest(unittest.TestCase):
 
         # Fill disk
         fh.seek(0)
-        for off in range(0, self.dev_size, self.obj_size):
+        for off in range(0, self.dev_size, bl):
             buf = self.get_data(bl)
             pwrite(self.s3, buf, offset=off, flags=0)
             fh.write(buf)
@@ -717,21 +709,22 @@ class LocalTest(unittest.TestCase):
         # Test different kinds of zero requests
         corner_cases = (
             1, 2,
-            bl-2, bl-1, bl+2,
+            bl-2, bl-1, bl, bl+2,
             2*bl-1, 2*bl, 2*bl+1,
             5*bl-5, 5*bl, 5*bl+5)
-        for off in (0,) + corner_cases:
-            for len_ in corner_cases:
-                zero(self.s3, len_, off, flags=0)
-                fh.seek(off)
-                fh.write(bytearray(len_))
-                self.compare_to_ref()
+        for flags in (0, nbdkit.FLAG_MAY_TRIM):
+            for off in (0,) + corner_cases:
+                for len_ in corner_cases:
+                    zero(self.s3, len_, off, flags=flags)
+                    fh.seek(off)
+                    fh.write(bytearray(len_))
+                    self.compare_to_ref()
 
-                # Re-fill with data
-                buf = self.get_data(len_)
-                pwrite(self.s3, buf, off, flags=0)
-                fh.seek(off)
-                fh.write(buf)
+                    # Re-fill with data
+                    buf = self.get_data(len_)
+                    pwrite(self.s3, buf, off, flags=0)
+                    fh.seek(off)
+                    fh.write(buf)
 
     def test_trim(self):
         bl = self.obj_size
@@ -786,14 +779,13 @@ class RemoteTest(unittest.TestCase):
 
         config_complete()
         self.s3 = open(False)
-        self.rnd_fh = builtins.open('/dev/urandom', 'rb')
 
     def tearDown(self) -> None:
         super().tearDown()
-        self.rnd_fh.close()
 
-    def get_data(self, len):
-        buf = self.rnd_fh.read(len // 2 + 1)
+    @staticmethod
+    def get_data(len):
+        buf = secrets.token_bytes(len // 2 + 1)
         return base64.b16encode(buf)[:len]
 
     def test_zero(self):
