@@ -211,8 +211,8 @@ expr (enum expr_type t, ...)
   expr_t e = { .t = t };
   va_list args;
 
-  /* Note that you cannot pass uint8_t through varargs, so for the
-   * byte fields we use int here.
+  /* Note that for varargs, uint8_t is automatically promoted to int
+   * (note: signed), so for the byte fields we use int here.
    */
   va_start (args, t);
   switch (t) {
@@ -640,7 +640,7 @@ parser (int level, const char *value, size_t *start, size_t len,
       }
       id = list.ptr[list.len-1];
       list.len--;
-      APPEND_EXPR (new_node (expr (EXPR_SLICE, id, i64, m)));
+      APPEND_EXPR (new_node (expr (EXPR_SLICE, id, (uint64_t) i64, m)));
       break;
 
     case '<':
