@@ -112,36 +112,57 @@ static enum { cache_default, cache_none } cache_mode = cache_default;
 struct write_window {
   int fd;
   uint64_t offset;
-  size_t len;
+  size_t len;           /* window slot only valid if len > 0 */
 };
 
 static pthread_mutex_t window_lock = PTHREAD_MUTEX_INITIALIZER;
 static struct write_window window[NR_WINDOWS];
 
-static void
+static int
 evict_writes (int fd, uint64_t offset, size_t len)
 {
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
+  struct write_window oldest = { 0 };
 
-  /* Evict the oldest window from the page cache. */
-  if (window[0].len > 0) {
-    sync_file_range (window[0].fd, window[0].offset, window[0].len,
-                     SYNC_FILE_RANGE_WAIT_BEFORE|SYNC_FILE_RANGE_WRITE|
-                     SYNC_FILE_RANGE_WAIT_AFTER);
-    posix_fadvise (window[0].fd, window[0].offset, window[0].len,
-                   POSIX_FADV_DONTNEED);
+  {
+    ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
+
+    /* Save oldest window[0] for eviction below, and move all windows
+     * down one.  Set the newest slot to empty.
+     */
+    oldest = window[0];
+    memmove (&window[0], &window[1], sizeof window[0] * (NR_WINDOWS-1));
+    window[NR_WINDOWS-1].len = 0;
+
+    /* Tell Linux to start writing the current range out to disk
+     * (asynchronously).
+     */
+    if (sync_file_range (fd, offset, len, SYNC_FILE_RANGE_WRITE) == -1) {
+      nbdkit_error ("sync_file_range: cache=none: starting eviction: %m");
+      return -1;
+    }
+
+    /* Add the range to the newest end of the list of windows. */
+    window[NR_WINDOWS-1].fd = fd;
+    window[NR_WINDOWS-1].offset = offset;
+    window[NR_WINDOWS-1].len = len;
+  }
+  /* Release lock here. */
+
+  /* Evict the oldest window from the page cache (synchronously). */
+  if (oldest.len > 0) {
+    if (sync_file_range (oldest.fd, oldest.offset, oldest.len,
+                         SYNC_FILE_RANGE_WAIT_BEFORE |
+                         SYNC_FILE_RANGE_WRITE |
+                         SYNC_FILE_RANGE_WAIT_AFTER) == -1) {
+      nbdkit_error ("sync_file_range: cache=none: evicting oldest window: %m");
+      return -1;
+    }
+    if (posix_fadvise (oldest.fd, oldest.offset, oldest.len,
+                       POSIX_FADV_DONTNEED) == -1)
+      nbdkit_debug ("posix_fadvise: POSIX_FADV_DONTNEED: (ignored): %m");
   }
 
-  /* Move the Nth window to N-1. */
-  memmove (&window[0], &window[1], sizeof window[0] * (NR_WINDOWS-1));
-
-  /* Set up the current window and tell Linux to start writing it out
-   * to disk (asynchronously).
-   */
-  sync_file_range (fd, offset, len, SYNC_FILE_RANGE_WRITE);
-  window[NR_WINDOWS-1].fd = fd;
-  window[NR_WINDOWS-1].offset = offset;
-  window[NR_WINDOWS-1].len = len;
+  return 0;
 }
 
 /* When we close the handle we must remove any windows which are still
@@ -383,6 +404,9 @@ file_dump_plugin (void)
 #ifdef BLKSSZGET
   printf ("file_blksszget=yes\n");
 #endif
+#ifdef BLKDISCARD
+  printf ("file_blkdiscard=yes\n");
+#endif
 #ifdef BLKZEROOUT
   printf ("file_blkzeroout=yes\n");
 #endif
@@ -485,7 +509,8 @@ struct handle {
   bool can_punch_hole;
   bool can_zero_range;
   bool can_fallocate;
-  bool can_zeroout;
+  bool can_blkdiscard;
+  bool can_blkzeroout;
 };
 
 /* Common code for opening a file by name, used by mode_filename and
@@ -725,7 +750,8 @@ file_open (int readonly)
 #endif
 
   h->can_fallocate = true;
-  h->can_zeroout = h->is_block_device;
+  h->can_blkzeroout = h->is_block_device;
+  h->can_blkdiscard = h->is_block_device;
 
   return h;
 }
@@ -855,8 +881,8 @@ file_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
 {
   struct handle *h = handle;
 #if defined (HAVE_POSIX_FADVISE) && defined (POSIX_FADV_DONTNEED)
-  uint32_t orig_count = count;
-  uint64_t orig_offset = offset;
+  const uint32_t orig_count = count;
+  const uint64_t orig_offset = offset;
 #endif
 
   while (count > 0) {
@@ -889,10 +915,9 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
              uint32_t flags)
 {
   struct handle *h = handle;
-
 #if EVICT_WRITES
-  uint32_t orig_count = count;
-  uint64_t orig_offset = offset;
+  const uint32_t orig_count = count;
+  const uint64_t orig_offset = offset;
 #endif
 
   while (count > 0) {
@@ -910,21 +935,45 @@ file_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
     return -1;
 
 #if EVICT_WRITES
-  if (cache_mode == cache_none)
-    evict_writes (h->fd, orig_offset, orig_count);
+  if (cache_mode == cache_none) {
+    if (evict_writes (h->fd, orig_offset, orig_count) == -1)
+      return -1;
+  }
 #endif
 
   return 0;
 }
 
-#if defined (FALLOC_FL_PUNCH_HOLE) || defined (FALLOC_FL_ZERO_RANGE)
+#if defined(FALLOC_FL_PUNCH_HOLE) || defined(FALLOC_FL_ZERO_RANGE)
 static int
 do_fallocate (int fd, int mode_, off_t offset, off_t len)
 {
-  int r = fallocate (fd, mode_, offset, len);
+  int r;
+
+  r = fallocate (fd, mode_, offset, len);
+
+  if (file_debug_zero)
+    nbdkit_debug ("fallocate (["
+#if defined(FALLOC_FL_PUNCH_HOLE)
+                  "%s"
+#endif
+#if defined(FALLOC_FL_ZERO_RANGE)
+                  "%s"
+#endif
+                  " ], %" PRIu64 ", %" PRIu64") => %d (%d)",
+#if defined(FALLOC_FL_PUNCH_HOLE)
+                  mode_ & FALLOC_FL_PUNCH_HOLE ? " FALLOC_FL_PUNCH_HOLE" : "",
+#endif
+#if defined(FALLOC_FL_ZERO_RANGE)
+                  mode_ & FALLOC_FL_ZERO_RANGE ? " FALLOC_FL_ZERO_RANGE" : "",
+#endif
+                  (uint64_t) offset, (uint64_t) len, r,
+                  r == -1 ? errno : 0);
+
   if (r == -1 && errno == ENODEV) {
     /* kernel 3.10 fails with ENODEV for block device. Kernel >= 4.9 fails
-       with EOPNOTSUPP in this case. Normalize errno to simplify callers. */
+     * with EOPNOTSUPP in this case. Normalize errno to simplify callers.
+     */
     errno = EOPNOTSUPP;
   }
   return r;
@@ -936,9 +985,14 @@ static int
 file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
   struct handle *h __attribute__ ((unused)) = handle;
+  const bool may_trim __attribute__ ((unused)) = flags & NBDKIT_FLAG_MAY_TRIM;
 
+  /* These alternate zeroing methods are ordered.  Methods which can
+   * trim (if may_trim is set) are tried first.  Methods which can
+   * only zero are tried last.
+   */
 #ifdef FALLOC_FL_PUNCH_HOLE
-  if (h->can_punch_hole && (flags & NBDKIT_FLAG_MAY_TRIM)) {
+  if (may_trim && h->can_punch_hole) {
     int r;
 
     r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
@@ -959,6 +1013,51 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   }
 #endif
 
+#if defined(BLKDISCARD) && defined(FALLOC_FL_ZERO_RANGE)
+  /* For aligned range and block device, we can use BLKDISCARD to
+   * trim.  However BLKDISCARD doesn't necessarily zero (eg for local
+   * disk) so we have to zero first and then discard.
+   *
+   * In future all Linux block devices may understand
+   * FALLOC_FL_PUNCH_HOLE which means this case would no longer be
+   * necessary, since the case above will handle it.
+   */
+  if (may_trim && h->can_blkdiscard && h->can_zero_range &&
+      IS_ALIGNED (offset | count, h->sector_size)) {
+    int r;
+    uint64_t range[2] = {offset, count};
+
+    r = do_fallocate (h->fd, FALLOC_FL_ZERO_RANGE, offset, count);
+    if (r == 0) {
+      /* We could use FALLOC_FL_PUNCH_HOLE here instead, but currently
+       * thin LVs do not support it (XXX 2025-04).
+       */
+      r = ioctl (h->fd, BLKDISCARD, &range);
+      if (r == 0) {
+        if (file_debug_zero)
+          nbdkit_debug ("h->can_blkdiscard && may_trim && IS_ALIGNED: "
+                        "zero succeeded using BLKDISCARD");
+        goto out;
+      }
+
+      if (!is_enotsup (errno)) {
+        nbdkit_error ("zero: %m");
+        return -1;
+      }
+
+      h->can_blkdiscard = false;
+    }
+    else {
+      if (!is_enotsup (errno)) {
+        nbdkit_error ("zero: %m");
+        return -1;
+      }
+
+      h->can_fallocate = false;
+    }
+  }
+#endif
+
 #ifdef FALLOC_FL_ZERO_RANGE
   if (h->can_zero_range) {
     int r;
@@ -966,7 +1065,7 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
     r = do_fallocate (h->fd, FALLOC_FL_ZERO_RANGE, offset, count);
     if (r == 0) {
       if (file_debug_zero)
-        nbdkit_debug ("h->can_zero-range: "
+        nbdkit_debug ("h->can_zero_range: "
                       "zero succeeded using fallocate");
       goto out;
     }
@@ -981,9 +1080,10 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 #endif
 
 #ifdef FALLOC_FL_PUNCH_HOLE
-  /* If we can punch hole but may not trim, we can combine punching hole and
-   * fallocate to zero a range. This is expected to be more efficient than
-   * writing zeroes manually. */
+  /* If we can punch hole but may not trim, we can combine punching
+   * hole and fallocate to zero a range. This is expected to be more
+   * efficient than writing zeroes manually.
+   */
   if (h->can_punch_hole && h->can_fallocate) {
     int r;
 
@@ -1017,14 +1117,14 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 
 #ifdef BLKZEROOUT
   /* For aligned range and block device, we can use BLKZEROOUT. */
-  if (h->can_zeroout && IS_ALIGNED (offset | count, h->sector_size)) {
+  if (h->can_blkzeroout && IS_ALIGNED (offset | count, h->sector_size)) {
     int r;
     uint64_t range[2] = {offset, count};
 
     r = ioctl (h->fd, BLKZEROOUT, &range);
     if (r == 0) {
       if (file_debug_zero)
-        nbdkit_debug ("h->can_zeroout && IS_ALIGNED: "
+        nbdkit_debug ("h->can_blkzeroout && IS_ALIGNED: "
                       "zero succeeded using BLKZEROOUT");
       goto out;
     }
@@ -1034,7 +1134,7 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
       return -1;
     }
 
-    h->can_zeroout = false;
+    h->can_blkzeroout = false;
   }
 #endif
 
@@ -1057,25 +1157,42 @@ file_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 static int
 file_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
-#ifdef FALLOC_FL_PUNCH_HOLE
-  struct handle *h = handle;
-  int r;
+  struct handle *h __attribute__ ((unused)) = handle;
 
+#ifdef FALLOC_FL_PUNCH_HOLE
   if (h->can_punch_hole) {
+    int r;
+
     r = do_fallocate (h->fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
                       offset, count);
     if (r == -1) {
-      /* Trim is advisory; we don't care if it fails for anything other
-       * than EIO or EPERM. */
-      if (errno == EPERM || errno == EIO) {
+      if (!is_enotsup (errno)) {
         nbdkit_error ("fallocate: %m");
         return -1;
       }
 
-      if (is_enotsup (EOPNOTSUPP))
-        h->can_punch_hole = false;
+      h->can_punch_hole = false;
+    }
+  }
+#endif
 
-      nbdkit_debug ("ignoring failed fallocate during trim: %m");
+#ifdef BLKDISCARD
+  /* In future all Linux block devices may understand
+   * FALLOC_FL_PUNCH_HOLE which means this case would no longer be
+   * necessary, since the case above will handle it.
+   */
+  if (h->can_blkdiscard && IS_ALIGNED (offset | count, h->sector_size)) {
+    int r;
+    uint64_t range[2] = {offset, count};
+
+    r = ioctl (h->fd, BLKDISCARD, &range);
+    if (r == -1) {
+      if (!is_enotsup (errno)) {
+        nbdkit_error ("ioctl: BLKDISCARD: %m");
+        return -1;
+      }
+
+      h->can_blkdiscard = false;
     }
   }
 #endif
@@ -1113,7 +1230,7 @@ do_extents (void *handle, uint32_t count, uint64_t offset,
 {
   struct handle *h = handle;
   const bool req_one = flags & NBDKIT_FLAG_REQ_ONE;
-  uint64_t end = offset + count;
+  const uint64_t end = offset + count;
 
   do {
     off_t pos;
@@ -1185,7 +1302,7 @@ file_cache (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
   struct handle *h = handle;
   int r;
 
-  /* Cache is advisory, we don't care if this fails */
+  /* This function returns an errno. */
   r = posix_fadvise (h->fd, offset, count, POSIX_FADV_WILLNEED);
   if (r) {
     errno = r;
