@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -38,7 +38,7 @@
 
 #include "regions.h"
 
-extern char *dir;
+extern const char *dir;
 extern const char *label;
 extern const char *type;
 extern int64_t size;
@@ -76,11 +76,11 @@ struct virtual_disk {
 
 /* virtual-disk.c */
 extern void init_virtual_disk (struct virtual_disk *disk)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 extern int create_virtual_disk (struct virtual_disk *disk)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 extern void free_virtual_disk (struct virtual_disk *disk)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 
 /* partition-gpt.c */
 extern int create_partition_table (struct virtual_disk *disk);
