@@ -36,6 +36,7 @@
 //! ramdisk.rs in the source code for a complete example.
 //!
 //! ```no_run
+//! # use std::ffi::CStr;
 //! # use nbdkit::*;
 //! #[derive(Default)]
 //! struct MyPlugin {
@@ -48,8 +49,8 @@
 //!         # Ok(0)
 //!     }
 //!
-//!     fn name() -> &'static str {
-//!         "my_plugin"
+//!     fn name() -> &'static CStr {
+//!         c"my_plugin"
 //!     }
 //!
 //!     fn open(_readonly: bool) -> Result<Box<dyn Server>> {
@@ -146,19 +147,13 @@ mod unreachable {
 static mut AFTER_FORK: fn() -> Result<()> = unreachable::config_complete;
 static mut CONFIG: fn(k: &str, v: &str) -> Result<()> = unreachable::config;
 static mut CONFIG_COMPLETE: fn() -> Result<()> = unreachable::config_complete;
-static mut CONFIG_HELP: Vec<u8> = Vec::new();
-static mut DESCRIPTION: Vec<u8> = Vec::new();
 static mut DUMP_PLUGIN: fn() = unreachable::dump_plugin;
 static mut GET_READY: fn() -> Result<()> = unreachable::config_complete;
 static mut LOAD: fn() = unreachable::dump_plugin;
-static mut LONGNAME: Vec<u8> = Vec::new();
-static mut MAGIC_CONFIG_KEY: Vec<u8> = Vec::new();
-static mut NAME: Vec<u8> = Vec::new();
 static mut OPEN: fn(readonly: bool) -> Result<Box<dyn Server>> = unreachable::open;
 static mut PRECONNECT: fn(readonly: bool) -> Result<()> = unreachable::preconnect;
 static mut THREAD_MODEL: fn() -> Result<ThreadModel> = unreachable::thread_model;
 static mut UNLOAD: fn() = unreachable::dump_plugin;
-static mut VERSION: Vec<u8> = Vec::new();
 static INIT: Once = Once::new();
 
 bitflags! {
@@ -307,7 +302,7 @@ impl ExtentHandle {
         match r {
             0 => Ok(()),
             -1 => Err(io::Error::last_os_error().into()),
-            x => panic!("Undocumented return value {} from nbdkit_add_extent", x)
+            x => panic!("Undocumented return value {x} from nbdkit_add_extent")
         }
     }
 }
@@ -316,7 +311,7 @@ impl ExtentHandle {
 mod ffi {
     use super::*;
 
-    pub(super) extern fn after_fork() -> c_int {
+    pub(super) extern "C" fn after_fork() -> c_int {
         match unsafe { AFTER_FORK() } {
             Ok(()) => 0,
             Err(e) => {
@@ -326,10 +321,10 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn block_size(h: *mut c_void,
-                                    minp: *mut u32,
-                                    prefp: *mut u32,
-                                    maxp: *mut u32) -> c_int {
+    pub(super) extern "C" fn block_size(h: *mut c_void,
+                                        minp: *mut u32,
+                                        prefp: *mut u32,
+                                        maxp: *mut u32) -> c_int {
         let server = unsafe { downcast(h) };
         match server.block_size() {
             Err(e) => {
@@ -347,7 +342,7 @@ mod ffi {
 
     macro_rules! can_method {
         ( $meth:ident ) => {
-            pub(super) extern fn $meth(h: *mut c_void) -> c_int {
+            pub(super) extern "C" fn $meth(h: *mut c_void) -> c_int {
                 let server = unsafe { downcast(h) };
                 match server.$meth() {
                     Err(e) => {
@@ -362,10 +357,10 @@ mod ffi {
 
     macro_rules! trim_like {
         ( $meth:ident) => {
-            pub(super) extern fn $meth(h: *mut c_void,
-                                      count: u32,
-                                      offset: u64,
-                                      rawflags: u32) -> c_int
+            pub(super) extern "C" fn $meth(h: *mut c_void,
+                                           count: u32,
+                                           offset: u64,
+                                           rawflags: u32) -> c_int
             {
                 let server = unsafe { downcast(h) };
                 let flags = Flags::from_bits(rawflags)
@@ -381,10 +376,10 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn cache(h: *mut c_void,
-                               count: u32,
-                               offset: u64,
-                               _rawflags: u32) -> c_int
+    pub(super) extern "C" fn cache(h: *mut c_void,
+                                   count: u32,
+                                   offset: u64,
+                                   _rawflags: u32) -> c_int
     {
         let server = unsafe { downcast(h) };
         match server.cache(count, offset) {
@@ -396,7 +391,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn close(selfp: *mut c_void) {
+    pub(super) extern "C" fn close(selfp: *mut c_void) {
         unsafe {
             drop(Box::from_raw(selfp as *mut Box<dyn Server>));
         }
@@ -412,7 +407,8 @@ mod ffi {
     can_method!(can_trim);
     can_method!(can_zero);
 
-    pub(super) extern fn config(k: *const c_char, v: *const c_char) -> c_int {
+    pub(super) extern "C" fn config(k: *const c_char,
+                                    v: *const c_char) -> c_int {
         let key = match unsafe { CStr::from_ptr(k) }.to_str() {
             Ok(s) => s,
             Err(e) => {
@@ -438,7 +434,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn config_complete() -> c_int {
+    pub(super) extern "C" fn config_complete() -> c_int {
         match unsafe { CONFIG_COMPLETE() } {
             Ok(()) => 0,
             Err(e) => {
@@ -459,16 +455,16 @@ mod ffi {
         &**(p as *const Box<dyn Server>)
     }
 
-    pub(super) extern fn dump_plugin() {
+    pub(super) extern "C" fn dump_plugin() {
         unsafe { DUMP_PLUGIN() }
     }
 
     // TODO: document REQ_ONE
-    pub(super) extern fn extents(h: *mut c_void,
-                                 count: u32,
-                                 offset: u64,
-                                 rawflags: u32,
-                                 extents: *mut c_void ) -> c_int
+    pub(super) extern "C" fn extents(h: *mut c_void,
+                                     count: u32,
+                                     offset: u64,
+                                     rawflags: u32,
+                                     extents: *mut c_void ) -> c_int
     {
         let server = unsafe { downcast(h) };
         let mut exh = ExtentHandle(extents);
@@ -482,7 +478,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn flush(h: *mut c_void, _rawflags: u32) -> c_int {
+    pub(super) extern "C" fn flush(h: *mut c_void, _rawflags: u32) -> c_int {
         let server = unsafe { downcast(h) };
         match server.flush() {
             Ok(()) => 0,
@@ -493,7 +489,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn get_ready() -> c_int {
+    pub(super) extern "C" fn get_ready() -> c_int {
         match unsafe { GET_READY() } {
             Ok(()) => 0,
             Err(e) => {
@@ -503,7 +499,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn get_size(h: *mut c_void) -> i64 {
+    pub(super) extern "C" fn get_size(h: *mut c_void) -> i64 {
         let server = unsafe { downcast(h) };
         match server.get_size() {
             Ok(size) => size,
@@ -516,11 +512,11 @@ mod ffi {
 
     can_method!(is_rotational);
 
-    pub(super) extern fn load() {
+    pub(super) extern "C" fn load() {
         unsafe { LOAD() }
     }
 
-    pub(super) extern fn open(readonly: c_int) -> *mut c_void {
+    pub(super) extern "C" fn open(readonly: c_int) -> *mut c_void {
         // We need to double-box to turn the trait object (fat pointer) into a
         // thin pointer
         match unsafe { OPEN(readonly != 0) } {
@@ -534,11 +530,11 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn pread(h: *mut c_void,
-                               bufp: *mut c_char,
-                               count: u32,
-                               offset: u64,
-                               _flags: u32) -> c_int
+    pub(super) extern "C" fn pread(h: *mut c_void,
+                                   bufp: *mut c_char,
+                                   count: u32,
+                                   offset: u64,
+                                   _flags: u32) -> c_int
     {
         let server = unsafe { downcast(h) };
         let buf = unsafe {
@@ -553,7 +549,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn preconnect(readonly: c_int) -> c_int {
+    pub(super) extern "C" fn preconnect(readonly: c_int) -> c_int {
         match unsafe { PRECONNECT(readonly != 0) } {
             Ok(()) => 0,
             Err(e) => {
@@ -563,11 +559,11 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn pwrite(h: *mut c_void,
-                                bufp: *const c_char,
-                                count: u32,
-                                offset: u64,
-                                rawflags: u32) -> c_int
+    pub(super) extern "C" fn pwrite(h: *mut c_void,
+                                    bufp: *const c_char,
+                                    count: u32,
+                                    offset: u64,
+                                    rawflags: u32) -> c_int
     {
         let server = unsafe { downcast(h) };
         let buf = unsafe {
@@ -592,7 +588,7 @@ mod ffi {
         }
     }
 
-    pub(super) extern fn thread_model() -> c_int {
+    pub(super) extern "C" fn thread_model() -> c_int {
         match unsafe { THREAD_MODEL() } {
             Ok(x) => x as c_int,
             Err(e) => {
@@ -604,7 +600,7 @@ mod ffi {
 
     trim_like!(trim);
 
-    pub(super) extern fn unload() {
+    pub(super) extern "C" fn unload() {
         unsafe { UNLOAD() }
     }
 
@@ -756,10 +752,10 @@ pub trait Server {
     /// This optional multi-line help message should summarize any key=value
     /// parameters that it takes. It does not need to repeat what already
     /// appears in [`Server::description`].
-    fn config_help() -> Option<&'static str> where Self: Sized { None }
+    fn config_help() -> Option<&'static CStr> where Self: Sized { None }
 
     /// An optional multi-line description of the plugin.
-    fn description() -> Option<&'static str> where Self: Sized { None }
+    fn description() -> Option<&'static CStr> where Self: Sized { None }
 
     /// During the data serving phase, this callback is used to detect
     /// allocated, sparse and zeroed regions of the disk.
@@ -826,7 +822,7 @@ pub trait Server {
 
     /// An optional free text name of the plugin. This field is used in error
     /// messages.
-    fn longname() -> Option<&'static str> where Self: Sized { None }
+    fn longname() -> Option<&'static CStr> where Self: Sized { None }
 
     /// This optional string can be used to set a "magic" key used when parsing
     /// plugin parameters. It affects how "bare parameters" (those which do not
@@ -839,13 +835,13 @@ pub trait Server {
     /// the first parameter on the command line is bare then it is passed to the
     /// `Server::config` method as: `config("script", value);`. Any other bare
     /// parameters give errors.
-    fn magic_config_key() -> Option<&'static str> where Self: Sized { None }
+    fn magic_config_key() -> Option<&'static CStr> where Self: Sized { None }
 
     /// The name of the plugin.
     ///
     /// It must contain only ASCII alphanumeric characters and be unique amongst
     /// all plugins.
-    fn name() -> &'static str where Self: Sized;
+    fn name() -> &'static CStr where Self: Sized;
 
     /// Allocate and return a new `Server` handle to the client.
     ///
@@ -908,7 +904,7 @@ pub trait Server {
 
     /// An optional version string which is displayed in help and debugging
     /// output.
-    fn version() -> Option<&'static str> where Self: Sized { None }
+    fn version() -> Option<&'static CStr> where Self: Sized { None }
 
     /// Write data to the backing store.
     ///
@@ -935,7 +931,7 @@ pub trait Server {
     /// * `count`:  Length of the region to write in bytes
     /// * `offset`: Offset of the region to write in the backing store.
     /// * `flags`:  May include [`Flags::MAY_TRIM`], [`Flags::FAST_ZERO`],
-    ///             and/or [`Flags::FUA`].
+    ///   and/or [`Flags::FUA`].
     fn zero(&self, count: u32, offset: u64, flags: Flags) -> Result<()> {
         unimplemented!()
     }
@@ -1001,46 +997,30 @@ impl Builder {
                 PRECONNECT = S::preconnect;
                 THREAD_MODEL = S::thread_model;
                 UNLOAD = S::unload;
-                NAME = CString::new(S::name()).unwrap().into_bytes_with_nul();
-                if let Some(s) = S::config_help() {
-                    CONFIG_HELP = CString::new(s).unwrap().into_bytes_with_nul();
-                }
-                if let Some(s) = S::description() {
-                    DESCRIPTION = CString::new(s).unwrap().into_bytes_with_nul();
-                }
-                if let Some(s) = S::longname() {
-                    LONGNAME = CString::new(s).unwrap().into_bytes_with_nul();
-                }
-                if let Some(s) = S::magic_config_key() {
-                    MAGIC_CONFIG_KEY = CString::new(s).unwrap()
-                        .into_bytes_with_nul();
-                }
-                if let Some(s) = S::version() {
-                    VERSION = CString::new(s).unwrap().into_bytes_with_nul();
-                }
             };
         });
 
+        let name = S::name().as_ptr();
         let config_help = S::config_help()
-            .map(|_| unsafe {CONFIG_HELP.as_ptr()} as *const c_char)
+            .map(CStr::as_ptr)
             .unwrap_or(ptr::null());
         let description = S::description()
-            .map(|_| unsafe {DESCRIPTION.as_ptr()} as *const c_char)
+            .map(CStr::as_ptr)
             .unwrap_or(ptr::null());
         let longname = S::longname()
-            .map(|_| unsafe {LONGNAME.as_ptr()} as *const c_char)
+            .map(CStr::as_ptr)
             .unwrap_or(ptr::null());
         let magic_config_key = S::magic_config_key()
-            .map(|_| unsafe {MAGIC_CONFIG_KEY.as_ptr()} as *const c_char)
+            .map(CStr::as_ptr)
             .unwrap_or(ptr::null());
         let version = S::version()
-            .map(|_| unsafe {VERSION.as_ptr()} as *const c_char)
+            .map(CStr::as_ptr)
             .unwrap_or(ptr::null());
         let plugin = Plugin {
             _struct_size: mem::size_of::<Plugin>() as u64,
             _api_version: 2,
             _thread_model: ThreadModel::Parallel as c_int,
-            name: unsafe{ NAME.as_ptr() } as *const c_char,
+            name,
             longname,
             version,
             description,
@@ -1291,85 +1271,85 @@ pub struct Plugin {
     pub version: *const c_char,
     pub description: *const c_char,
 
-    pub load: Option<extern fn ()>,
-    pub unload: Option<extern fn ()>,
+    pub load: Option<extern "C" fn ()>,
+    pub unload: Option<extern "C" fn ()>,
 
-    pub config: Option<extern fn (*const c_char, *const c_char) -> c_int>,
-    pub config_complete: Option<extern fn () -> c_int>,
+    pub config: Option<extern "C" fn (*const c_char, *const c_char) -> c_int>,
+    pub config_complete: Option<extern "C" fn () -> c_int>,
     pub config_help: *const c_char,
 
-    pub open: extern fn (c_int) -> *mut c_void,
-    pub close: extern fn (*mut c_void),
+    pub open: extern "C" fn (c_int) -> *mut c_void,
+    pub close: extern "C" fn (*mut c_void),
 
-    pub get_size: extern fn (*mut c_void) -> i64,
+    pub get_size: extern "C" fn (*mut c_void) -> i64,
 
-    pub can_write: Option<extern fn (*mut c_void) -> c_int>,
-    pub can_flush: Option<extern fn (*mut c_void) -> c_int>,
-    pub is_rotational: Option<extern fn (*mut c_void) -> c_int>,
-    pub can_trim: Option<extern fn (*mut c_void) -> c_int>,
+    pub can_write: Option<extern "C" fn (*mut c_void) -> c_int>,
+    pub can_flush: Option<extern "C" fn (*mut c_void) -> c_int>,
+    pub is_rotational: Option<extern "C" fn (*mut c_void) -> c_int>,
+    pub can_trim: Option<extern "C" fn (*mut c_void) -> c_int>,
 
     // Slots for old v1 API functions.
-    #[doc(hidden)] pub _pread_v1: Option<extern fn ()>,
-    #[doc(hidden)] pub _pwrite_v1: Option<extern fn ()>,
-    #[doc(hidden)] pub _flush_v1: Option<extern fn ()>,
-    #[doc(hidden)] pub _trim_v1: Option<extern fn ()>,
-    #[doc(hidden)] pub _zero_v1: Option<extern fn ()>,
+    #[doc(hidden)] pub _pread_v1: Option<extern "C" fn ()>,
+    #[doc(hidden)] pub _pwrite_v1: Option<extern "C" fn ()>,
+    #[doc(hidden)] pub _flush_v1: Option<extern "C" fn ()>,
+    #[doc(hidden)] pub _trim_v1: Option<extern "C" fn ()>,
+    #[doc(hidden)] pub _zero_v1: Option<extern "C" fn ()>,
 
     #[doc(hidden)] pub errno_is_preserved: c_int,
 
-    pub dump_plugin: Option<extern fn ()>,
+    pub dump_plugin: Option<extern "C" fn ()>,
 
-    pub can_zero: Option<extern fn (*mut c_void) -> c_int>,
-    pub can_fua: Option<extern fn (*mut c_void) -> c_int>,
+    pub can_zero: Option<extern "C" fn (*mut c_void) -> c_int>,
+    pub can_fua: Option<extern "C" fn (*mut c_void) -> c_int>,
 
-    pub pread: extern fn (h: *mut c_void, buf: *mut c_char, count: u32,
-                          offset: u64,
-                          flags: u32) -> c_int,
-    pub pwrite: Option<extern fn (h: *mut c_void, buf: *const c_char,
-                                  count: u32, offset: u64,
-                                  flags: u32) -> c_int>,
-    pub flush: Option<extern fn (h: *mut c_void, flags: u32) -> c_int>,
-    pub trim: Option<extern fn (h: *mut c_void,
-                                count: u32, offset: u64,
-                                flags: u32) -> c_int>,
-    pub zero: Option<extern fn (h: *mut c_void,
-                                count: u32, offset: u64,
-                                flags: u32) -> c_int>,
+    pub pread: extern "C" fn (h: *mut c_void, buf: *mut c_char, count: u32,
+                              offset: u64,
+                              flags: u32) -> c_int,
+    pub pwrite: Option<extern "C" fn (h: *mut c_void, buf: *const c_char,
+                                      count: u32, offset: u64,
+                                      flags: u32) -> c_int>,
+    pub flush: Option<extern "C" fn (h: *mut c_void, flags: u32) -> c_int>,
+    pub trim: Option<extern "C" fn (h: *mut c_void,
+                                    count: u32, offset: u64,
+                                    flags: u32) -> c_int>,
+    pub zero: Option<extern "C" fn (h: *mut c_void,
+                                    count: u32, offset: u64,
+                                    flags: u32) -> c_int>,
 
     pub magic_config_key: *const c_char,
 
-    pub can_multi_conn: Option<extern fn (h: *mut c_void) -> c_int>,
+    pub can_multi_conn: Option<extern "C" fn (h: *mut c_void) -> c_int>,
 
-    pub can_extents: Option<extern fn (h: *mut c_void) -> c_int>,
-    pub extents: Option<extern fn (h: *mut c_void,
-                                   count: u32,
-                                   offset: u64,
-                                   rawflags: u32,
-                                   extent_handle: *mut c_void) -> c_int>,
+    pub can_extents: Option<extern "C" fn (h: *mut c_void) -> c_int>,
+    pub extents: Option<extern "C" fn (h: *mut c_void,
+                                       count: u32,
+                                       offset: u64,
+                                       rawflags: u32,
+                                       extent_handle: *mut c_void) -> c_int>,
 
-    pub can_cache: Option<extern fn (h: *mut c_void) -> c_int>,
-    pub cache: Option<extern fn (h: *mut c_void,
-                                 count: u32, offset: u64,
-                                 flags: u32) -> c_int>,
+    pub can_cache: Option<extern "C" fn (h: *mut c_void) -> c_int>,
+    pub cache: Option<extern "C" fn (h: *mut c_void,
+                                     count: u32, offset: u64,
+                                     flags: u32) -> c_int>,
 
-    pub thread_model: Option<extern fn () -> c_int>,
+    pub thread_model: Option<extern "C" fn () -> c_int>,
 
-    pub can_fast_zero: Option<extern fn (h: *mut c_void) -> c_int>,
-    pub preconnect: Option<extern fn(readonly: c_int) -> c_int>,
-    pub get_ready: Option<extern fn() -> c_int>,
-    pub after_fork: Option<extern fn() -> c_int>,
+    pub can_fast_zero: Option<extern "C" fn (h: *mut c_void) -> c_int>,
+    pub preconnect: Option<extern "C" fn(readonly: c_int) -> c_int>,
+    pub get_ready: Option<extern "C" fn() -> c_int>,
+    pub after_fork: Option<extern "C" fn() -> c_int>,
 
     // These fields exist in the struct but have not been implemented
     // by the Rust bindings yet.
-    #[doc(hidden)] pub _list_exports: Option<extern fn ()>,
+    #[doc(hidden)] pub _list_exports: Option<extern "C" fn ()>,
     #[doc(hidden)] pub _default_export: *const c_char,
     #[doc(hidden)] pub _export_description: *const c_char,
-    #[doc(hidden)] pub _cleanup: Option<extern fn ()>,
+    #[doc(hidden)] pub _cleanup: Option<extern "C" fn ()>,
 
-    pub block_size: Option<extern fn(h: *mut c_void,
-                                     minp: *mut u32,
-                                     prefp: *mut u32,
-                                     maxp: *mut u32) -> c_int>,
+    pub block_size: Option<extern "C" fn(h: *mut c_void,
+                                         minp: *mut u32,
+                                         prefp: *mut u32,
+                                         maxp: *mut u32) -> c_int>,
 }
 
 /// Register your plugin with NBDKit.
@@ -1379,6 +1359,7 @@ pub struct Plugin {
 /// # Examples
 ///
 /// ```
+/// # use std::ffi::CStr;
 /// # use nbdkit::*;
 /// struct MyPlugin{
 ///     // ...
@@ -1389,8 +1370,8 @@ pub struct Plugin {
 ///         // ...
 ///     }
 ///
-///     fn name() -> &'static str {
-///         "my_plugin"
+///     fn name() -> &'static CStr {
+///         c"my_plugin"
 ///     }
 ///
 ///     fn open(_readonly: bool) -> Result<Box<dyn Server>> {
@@ -1414,7 +1395,7 @@ pub struct Plugin {
 macro_rules! plugin {
     ( $cls:path { $($feat:ident),* } ) => {
         #[no_mangle]
-        pub extern fn plugin_init () -> *const ::nbdkit::Plugin {
+        pub extern "C" fn plugin_init () -> *const ::nbdkit::Plugin {
             let mut plugin = ::nbdkit::Builder::new();
             $(plugin.$feat = true;)*
             plugin.into_ptr::<$cls>()
@@ -1448,8 +1429,8 @@ mod t {
         }
 
         #[no_mangle]
-        extern fn nbdkit_peer_name( addr: *mut libc::sockaddr,
-                                    addrlen: *mut libc::socklen_t) -> c_int
+        extern "C" fn nbdkit_peer_name( addr: *mut libc::sockaddr,
+                                        addrlen: *mut libc::socklen_t) -> c_int
         {
             MockNbdkit::peer_name(addr, addrlen)
         }
