@@ -186,7 +186,11 @@ tcl_config_complete (void)
   int r;
   Tcl_Obj *cmd;
 
-  if (proc_defined ("config_complete")) {
+  if (!script) {
+    nbdkit_error ("the first parameter must be script=/path/to/script.tcl");
+    return -1;
+  }
+  else if (proc_defined ("config_complete")) {
     cmd = Tcl_NewObj ();
     Tcl_IncrRefCount (cmd);
     Tcl_ListObjAppendElement (0, cmd, Tcl_NewStringObj ("config_complete", -1));
@@ -277,7 +281,11 @@ tcl_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
   int r;
   Tcl_Obj *h = handle, *cmd, *res;
   unsigned char *res_bin;
+#ifdef TCL_SIZE_MAX /* Tcl >= 9 added new Tcl_Size typedef */
+  Tcl_Size res_len;
+#else
   int res_len;
+#endif
 
   cmd = Tcl_NewObj ();
   Tcl_IncrRefCount (cmd);
