@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,6 +43,9 @@
 
 typedef uint64_t VixError;
 #define VIX_OK 0
+#define VIX_E_FAIL 1
+#define VIX_E_NOT_SUPPORTED 6
+#define VIX_ASYNC 25000
 
 #define VIXDISKLIB_FLAG_OPEN_UNBUFFERED 1
 #define VIXDISKLIB_FLAG_OPEN_SINGLE_LINK 2
@@ -53,6 +56,28 @@ typedef uint64_t VixError;
 
 #define VIXDISKLIB_SECTOR_SIZE 512
 
+enum VixDiskLibDiskType {
+  VIXDISKLIB_DISK_MONOLITHIC_SPARSE = 1,
+  VIXDISKLIB_DISK_MONOLITHIC_FLAT = 2,
+  VIXDISKLIB_DISK_SPLIT_SPARSE = 3,
+  VIXDISKLIB_DISK_SPLIT_FLAT = 4,
+  VIXDISKLIB_DISK_VMFS_FLAT = 5,
+  VIXDISKLIB_DISK_STREAM_OPTIMIZED = 6,
+  VIXDISKLIB_DISK_VMFS_THIN = 7,
+  VIXDISKLIB_DISK_VMFS_SPARSE = 8
+};
+
+#define VIXDISKLIB_HWVERSION_WORKSTATION_4 3
+#define VIXDISKLIB_HWVERSION_WORKSTATION_5 4
+#define VIXDISKLIB_HWVERSION_WORKSTATION_6 6
+#define VIXDISKLIB_HWVERSION_ESX30 4
+#define VIXDISKLIB_HWVERSION_ESX4X 7
+#define VIXDISKLIB_HWVERSION_ESX50 8
+#define VIXDISKLIB_HWVERSION_ESX51 9
+#define VIXDISKLIB_HWVERSION_ESX55 10
+#define VIXDISKLIB_HWVERSION_ESX60 11
+#define VIXDISKLIB_HWVERSION_ESX65 13
+
 #define VIXDISKLIB_MIN_CHUNK_SIZE 128
 #define VIXDISKLIB_MAX_CHUNK_NUMBER (512*1024)
 
@@ -61,6 +86,8 @@ typedef void *VixDiskLibHandle;
 
 typedef void VixDiskLibGenericLogFunc (const char *fmt, va_list args);
 
+typedef void (*VixDiskLibCompletionCB) (void *data, VixError result);
+
 enum VixDiskLibCredType {
   VIXDISKLIB_CRED_UID       = 1,
   VIXDISKLIB_CRED_SESSIONID = 2,
@@ -145,4 +172,13 @@ typedef struct {
   VixDiskLibBlock blocks[1];
 } VixDiskLibBlockList;
 
+typedef struct {
+  enum VixDiskLibDiskType diskType;
+  enum VixDiskLibAdapterType adapterType;
+  uint16_t hwVersion;
+  uint64_t capacity;
+  uint32_t logicalSectorSize;
+  uint32_t physicalSectorSize;
+} VixDiskLibCreateParams;
+
 #endif /* NBDKIT_VDDK_STRUCTS_H */
