@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2019-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -56,7 +56,7 @@
 #define MAX_EXTENTS (1 * 1024 * 1024)
 
 /* Appendable list of extents. */
-DEFINE_VECTOR_TYPE(extents, struct nbdkit_extent);
+DEFINE_VECTOR_TYPE (extents, struct nbdkit_extent);
 
 struct nbdkit_extents {
   extents extents;
@@ -71,7 +71,7 @@ struct nbdkit_extents {
   int64_t next;
 };
 
-struct nbdkit_extents *
+NBDKIT_DLL_PUBLIC struct nbdkit_extents *
 nbdkit_extents_new (uint64_t start, uint64_t end)
 {
   struct nbdkit_extents *r;
@@ -105,7 +105,7 @@ nbdkit_extents_new (uint64_t start, uint64_t end)
   return r;
 }
 
-void
+NBDKIT_DLL_PUBLIC void
 nbdkit_extents_free (struct nbdkit_extents *exts)
 {
   if (exts) {
@@ -114,16 +114,16 @@ nbdkit_extents_free (struct nbdkit_extents *exts)
   }
 }
 
-size_t
+NBDKIT_DLL_PUBLIC size_t
 nbdkit_extents_count (const struct nbdkit_extents *exts)
 {
-  return exts->extents.size;
+  return exts->extents.len;
 }
 
-struct nbdkit_extent
+NBDKIT_DLL_PUBLIC struct nbdkit_extent
 nbdkit_get_extent (const struct nbdkit_extents *exts, size_t i)
 {
-  assert (i < exts->extents.size);
+  assert (i < exts->extents.len);
   return exts->extents.ptr[i];
 }
 
@@ -139,7 +139,7 @@ append_extent (struct nbdkit_extents *exts, const struct nbdkit_extent *e)
   return 0;
 }
 
-int
+NBDKIT_DLL_PUBLIC int
 nbdkit_add_extent (struct nbdkit_extents *exts,
                    uint64_t offset, uint64_t length, uint32_t type)
 {
@@ -160,7 +160,7 @@ nbdkit_add_extent (struct nbdkit_extents *exts,
     return 0;
 
   /* Ignore extents beyond the end of the range, or if list is full. */
-  if (offset >= exts->end || exts->extents.size >= MAX_EXTENTS)
+  if (offset >= exts->end || exts->extents.len >= MAX_EXTENTS)
     return 0;
 
   /* Shorten extents that overlap the end of the range. */
@@ -169,7 +169,7 @@ nbdkit_add_extent (struct nbdkit_extents *exts,
     length -= overlap;
   }
 
-  if (exts->extents.size == 0) {
+  if (exts->extents.len == 0) {
     /* If there are no existing extents, and the new extent is
      * entirely before start, ignore it.
      */
@@ -196,10 +196,10 @@ nbdkit_add_extent (struct nbdkit_extents *exts,
   }
 
   /* If we get here we are going to either add or extend. */
-  if (exts->extents.size > 0 &&
-      exts->extents.ptr[exts->extents.size-1].type == type) {
+  if (exts->extents.len > 0 &&
+      exts->extents.ptr[exts->extents.len-1].type == type) {
     /* Coalesce with the last extent. */
-    exts->extents.ptr[exts->extents.size-1].length += length;
+    exts->extents.ptr[exts->extents.len-1].length += length;
     return 0;
   }
   else {
@@ -211,28 +211,28 @@ nbdkit_add_extent (struct nbdkit_extents *exts,
 }
 
 /* Compute aligned extents on behalf of a filter. */
-int
-nbdkit_extents_aligned (struct nbdkit_next_ops *next_ops,
-                        nbdkit_backend *nxdata,
+NBDKIT_DLL_PUBLIC int
+nbdkit_extents_aligned (struct context *next_c,
                         uint32_t count, uint64_t offset,
                         uint32_t flags, uint32_t align,
                         struct nbdkit_extents *exts, int *err)
 {
+  struct nbdkit_next_ops *next = &next_c->next;
   size_t i;
   struct nbdkit_extent *e, *e2;
 
-  assert (IS_ALIGNED(count | offset, align));
+  assert (IS_ALIGNED (count | offset, align));
 
   /* Perform an initial query, then scan for the first unaligned extent. */
-  if (next_ops->extents (nxdata, count, offset, flags, exts, err) == -1)
+  if (next->extents (next_c, count, offset, flags, exts, err) == -1)
     return -1;
-  for (i = 0; i < exts->extents.size; ++i) {
+  for (i = 0; i < exts->extents.len; ++i) {
     e = &exts->extents.ptr[i];
-    if (!IS_ALIGNED(e->length, align)) {
+    if (!IS_ALIGNED (e->length, align)) {
       /* If the unalignment is past align, just truncate and return early */
       if (e->offset + e->length > offset + align) {
         e->length = ROUND_DOWN (e->length, align);
-        exts->extents.size = i + !!e->length;
+        exts->extents.len = i + !!e->length;
         exts->next = e->offset + e->length;
         break;
       }
@@ -249,7 +249,7 @@ nbdkit_extents_aligned (struct nbdkit_next_ops *next_ops,
        */
       assert (i == 0);
       while (e->length < align) {
-        if (exts->extents.size > 1) {
+        if (exts->extents.len > 1) {
           e->length += exts->extents.ptr[1].length;
           e->type &= exts->extents.ptr[1].type;
           extents_remove (&exts->extents, 1);
@@ -267,10 +267,10 @@ nbdkit_extents_aligned (struct nbdkit_next_ops *next_ops,
             *err = errno;
             return -1;
           }
-          if (next_ops->extents (nxdata, align - e->length,
-                                 offset + e->length,
-                                 flags & ~NBDKIT_FLAG_REQ_ONE,
-                                 extents2, err) == -1)
+          if (next->extents (next_c, align - e->length,
+                             offset + e->length,
+                             flags & ~NBDKIT_FLAG_REQ_ONE,
+                             extents2, err) == -1)
             return -1;
           e2 = &extents2->extents.ptr[0];
           assert (e2->offset == e->offset + e->length);
@@ -284,7 +284,7 @@ nbdkit_extents_aligned (struct nbdkit_next_ops *next_ops,
         }
       }
       e->length = align;
-      exts->extents.size = 1;
+      exts->extents.len = 1;
       exts->next = e->offset + e->length;
       break;
     }
@@ -292,3 +292,58 @@ nbdkit_extents_aligned (struct nbdkit_next_ops *next_ops,
   /* Once we get here, all extents are aligned. */
   return 0;
 }
+
+/* This is a convenient wrapper around next_ops->extents which can be
+ * used from filters where you want to get a complete set of extents
+ * covering the region [offset..offset+count-1].
+ */
+NBDKIT_DLL_PUBLIC struct nbdkit_extents *
+nbdkit_extents_full (struct context *next_c,
+                     uint32_t count, uint64_t offset, uint32_t flags,
+                     int *err)
+{
+  struct nbdkit_next_ops *next = &next_c->next;
+  struct nbdkit_extents *ret;
+
+  /* Clear REQ_ONE to ask the plugin for as much information as it is
+   * willing to return (the plugin may still truncate if it is too
+   * costly to provide everything).
+   */
+  flags &= ~NBDKIT_FLAG_REQ_ONE;
+
+  ret = nbdkit_extents_new (offset, offset+count);
+  if (ret == NULL) goto error0;
+
+  while (count > 0) {
+    const uint64_t old_offset = offset;
+    size_t i;
+
+    CLEANUP_EXTENTS_FREE struct nbdkit_extents *t
+      = nbdkit_extents_new (offset, offset+count);
+    if (t == NULL) goto error1;
+
+    if (next->extents (next_c, count, offset, flags, t, err) == -1)
+      goto error0;
+
+    for (i = 0; i < nbdkit_extents_count (t); ++i) {
+      const struct nbdkit_extent e = nbdkit_get_extent (t, i);
+      if (nbdkit_add_extent (ret, e.offset, e.length, e.type) == -1)
+        goto error1;
+
+      assert (e.length <= count);
+      offset += e.length;
+      count -= e.length;
+    }
+
+    /* If the plugin is behaving we must make forward progress. */
+    assert (offset > old_offset);
+  }
+
+  return ret;
+
+ error1:
+  *err = errno;
+ error0:
+  nbdkit_extents_free (ret);
+  return NULL;
+}
