@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -42,10 +42,12 @@
 
 #include <pthread.h>
 
+#include "nbdkit-plugin.h" /* only for NBDKIT_DLL_PUBLIC */
+
 #include "vddk-structs.h"
 
-#define STUB(fn,ret,args) extern ret fn args;
-#define OPTIONAL_STUB(fn,ret,args)
+#define STUB(fn, ret, args) extern ret fn args;
+#define OPTIONAL_STUB(fn, ret, args)
 #include "vddk-stubs.h"
 #undef STUB
 #undef OPTIONAL_STUB
@@ -64,7 +66,7 @@ bg_thread (void *datav)
   return NULL;
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_InitEx (uint32_t major, uint32_t minor,
                    VixDiskLibGenericLogFunc *log_function,
                    VixDiskLibGenericLogFunc *warn_function,
@@ -88,32 +90,40 @@ VixDiskLib_InitEx (uint32_t major, uint32_t minor,
   return VIX_OK;
 }
 
-void
+NBDKIT_DLL_PUBLIC void
 VixDiskLib_Exit (void)
 {
-  /* Do nothing. */
+  /* Stop the background thread. */
+  pthread_cancel (thread);
+  pthread_join (thread, NULL);
 }
 
-char *
+NBDKIT_DLL_PUBLIC const char *
+VixDiskLib_ListTransportModes (void)
+{
+  return "file:nbdssl:nbd";
+}
+
+NBDKIT_DLL_PUBLIC char *
 VixDiskLib_GetErrorText (VixError err, const char *unused)
 {
   return strdup ("dummy-vddk: error message");
 }
 
-void
+NBDKIT_DLL_PUBLIC void
 VixDiskLib_FreeErrorText (char *text)
 {
   free (text);
 }
 
-void
+NBDKIT_DLL_PUBLIC void
 VixDiskLib_FreeConnectParams (VixDiskLibConnectParams *params)
 {
   /* never called since we don't define optional AllocateConnectParams */
   abort ();
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_ConnectEx (const VixDiskLibConnectParams *params,
                       char read_only,
                       const char *snapshot_ref,
@@ -129,7 +139,7 @@ VixDiskLib_ConnectEx (const VixDiskLibConnectParams *params,
   return VIX_OK;
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_Open (const VixDiskLibConnection connection,
                  const char *path,
                  uint32_t flags,
@@ -142,25 +152,25 @@ VixDiskLib_Open (const VixDiskLibConnection connection,
   return VIX_OK;
 }
 
-const char *
+NBDKIT_DLL_PUBLIC const char *
 VixDiskLib_GetTransportMode (VixDiskLibHandle handle)
 {
   return "file";
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_Close (VixDiskLibHandle handle)
 {
   return VIX_OK;
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_Disconnect (VixDiskLibConnection connection)
 {
   return VIX_OK;
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_GetInfo (VixDiskLibHandle handle,
                     VixDiskLibInfo **info)
 {
@@ -169,13 +179,13 @@ VixDiskLib_GetInfo (VixDiskLibHandle handle,
   return VIX_OK;
 }
 
-void
+NBDKIT_DLL_PUBLIC void
 VixDiskLib_FreeInfo (VixDiskLibInfo *info)
 {
   free (info);
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_Read (VixDiskLibHandle handle,
                  uint64_t start_sector, uint64_t nr_sectors,
                  unsigned char *buf)
@@ -186,7 +196,20 @@ VixDiskLib_Read (VixDiskLibHandle handle,
   return VIX_OK;
 }
 
-VixError
+NBDKIT_DLL_PUBLIC VixError
+VixDiskLib_ReadAsync (VixDiskLibHandle handle,
+                      uint64_t start_sector, uint64_t nr_sectors,
+                      unsigned char *buf,
+                      VixDiskLibCompletionCB callback, void *data)
+{
+  size_t offset = start_sector * VIXDISKLIB_SECTOR_SIZE;
+
+  memcpy (buf, disk + offset, nr_sectors * VIXDISKLIB_SECTOR_SIZE);
+  callback (data, VIX_OK);
+  return VIX_ASYNC;
+}
+
+NBDKIT_DLL_PUBLIC VixError
 VixDiskLib_Write (VixDiskLibHandle handle,
                   uint64_t start_sector, uint64_t nr_sectors,
                   const unsigned char *buf)
@@ -196,3 +219,38 @@ VixDiskLib_Write (VixDiskLibHandle handle,
   memcpy (disk + offset, buf, nr_sectors * VIXDISKLIB_SECTOR_SIZE);
   return VIX_OK;
 }
+
+NBDKIT_DLL_PUBLIC VixError
+VixDiskLib_WriteAsync (VixDiskLibHandle handle,
+                       uint64_t start_sector, uint64_t nr_sectors,
+                       const unsigned char *buf,
+                       VixDiskLibCompletionCB callback, void *data)
+{
+  size_t offset = start_sector * VIXDISKLIB_SECTOR_SIZE;
+
+  memcpy (disk + offset, buf, nr_sectors * VIXDISKLIB_SECTOR_SIZE);
+  callback (data, VIX_OK);
+  return VIX_ASYNC;
+}
+
+NBDKIT_DLL_PUBLIC VixError
+VixDiskLib_Flush (VixDiskLibHandle handle)
+{
+  return VIX_OK;
+}
+
+NBDKIT_DLL_PUBLIC VixError
+VixDiskLib_Wait (VixDiskLibHandle handle)
+{
+  return VIX_OK;
+}
+
+NBDKIT_DLL_PUBLIC VixError
+VixDiskLib_Create (const VixDiskLibConnection connection,
+                   const char *path,
+                   const VixDiskLibCreateParams *create_params,
+                   void *progress_function_unused,
+                   void *progress_data_unused)
+{
+  return VIX_E_NOT_SUPPORTED;
+}
