@@ -41,18 +41,15 @@
 
 #include <libnbd.h>
 
+#include "requires.h"
+
 int
 main (int argc, char *argv[])
 {
   struct nbd_handle *nbd;
   int64_t size;
 
-  if (system ("nbdkit --exit-with-parent --version") != 0) {
-    printf ("%s: --exit-with-parent is not implemented on this platform, "
-            "skipping\n",
-            argv[0]);
-    exit (77);
-  }
+  requires ("nbdkit --exit-with-parent --version");
 
   nbd = nbd_create ();
   if (nbd == NULL) {
