@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2017-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -36,4 +36,5 @@ set -e
 # We only really care that the plugin compiled, so we don't
 # need to run it properly.
 libs=./.libs
-nbdkit --filter=$libs/test-cxx-filter.so $libs/test-cxx-plugin.so --version
+nbdkit --filter=$libs/test-cxx-filter.$SOEXT \
+       $libs/test-cxx-plugin.$SOEXT --version
