@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2017 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -46,8 +46,11 @@
 #include <sys/wait.h>
 #endif
 
+#include "requires.h"
 #include "test.h"
 
+#ifndef WIN32
+
 static char pidpath[] = "/tmp/nbdkitpidXXXXXX";
 
 static void run_test (void);
@@ -55,12 +58,7 @@ static void run_test (void);
 int
 main (int argc, char *argv[])
 {
-  if (system ("nbdkit --exit-with-parent --version") != 0) {
-    printf ("%s: --exit-with-parent is not implemented on this platform, "
-            "skipping\n",
-            argv[0]);
-    exit (77);
-  }
+  requires ("nbdkit --exit-with-parent --version");
 
   run_test ();
   exit (EXIT_SUCCESS);
@@ -216,3 +214,19 @@ run_test (void)
   printf ("monitor: success: nbdkit exited with parent\n");
   fflush (stdout);
 }
+
+#else /* WIN32 */
+
+/* The test above relies on Unixisms like fork so it won't work on
+ * Windows as written.  Since we don't implement --exit-with-parent
+ * anyway, bail for now and worry about it later.
+ */
+
+int
+main (int argc, char *argv[])
+{
+  skip_because ("neither --exit-with-parent nor this test "
+                "are implemented on Windows");
+}
+
+#endif /* WIN32 */
