@@ -34,13 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
-requires qemu-img --version
-
-# Does the nbdkit binary support TLS?
-if ! nbdkit --dump-config | grep -sq tls=yes; then
-    echo "$0: nbdkit built without TLS support"
-    exit 77
-fi
+requires "$QEMU_IMG" --version
+requires_tls_certificates
 
 # Does the nbd plugin support TLS?
 if ! nbdkit --dump-plugin nbd | grep -sq libnbd_tls=1; then
@@ -48,14 +43,6 @@ if ! nbdkit --dump-plugin nbd | grep -sq libnbd_tls=1; then
     exit 77
 fi
 
-# Did we create the PKI files?
-# Probably 'certtool' is missing.
-pkidir="$PWD/pki"
-if [ ! -f "$pkidir/ca-cert.pem" ]; then
-    echo "$0: PKI files were not created by the test harness"
-    exit 77
-fi
-
 sock1=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 sock2=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
 pid1="test-nbd-tls.pid1"
@@ -77,7 +64,7 @@ start_nbdkit -P "$pid1" -U "$sock1" \
     example1
 
 # Run unencrypted client
-qemu-img info --output=json -f raw "nbd+unix:///?socket=$sock2" > nbd-tls.out
+"$QEMU_IMG" info --output=json -f raw "nbd+unix:///?socket=$sock2" > nbd-tls.out
 
 cat nbd-tls.out
 
