@@ -57,7 +57,7 @@ timeout 30s </dev/null qemu-io -f raw -c "aio_write -P 1 0 512" \
 # With --threads=1, the write should complete first because it was issued first
 nbdkit -v -t 1 --filter=delay file test-parallel-file.data \
   wdelay=2 rdelay=1 --run 'timeout 60s </dev/null qemu-io -f raw \
-    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush $nbd' |
+    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush "$uri"' |
     tee test-parallel-file.out
 if test "$(grep '512/512' test-parallel-file.out)" != \
 "wrote 512/512 bytes at offset 512
@@ -68,7 +68,7 @@ fi
 # With default --threads, the faster read should complete first
 nbdkit -v --filter=delay file test-parallel-file.data \
   wdelay=2 rdelay=1 --run 'timeout 60s </dev/null qemu-io -f raw \
-    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush $nbd' |
+    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush "$uri"' |
     tee test-parallel-file.out
 if test "$(grep '512/512' test-parallel-file.out)" != \
 "read 512/512 bytes at offset 0
@@ -79,7 +79,7 @@ fi
 # With --filter=noparallel, the write should complete first because it was issued first
 nbdkit -v --filter=noparallel --filter=delay file test-parallel-file.data \
   wdelay=2 rdelay=1 --run 'timeout 60s </dev/null qemu-io -f raw \
-    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush $nbd' |
+    -c "aio_write -P 2 512 512" -c "aio_read -P 1 0 512" -c aio_flush "$uri"' |
     tee test-parallel-file.out
 if test "$(grep '512/512' test-parallel-file.out)" != \
 "wrote 512/512 bytes at offset 512
