@@ -45,6 +45,7 @@
 #include "cleanup.h"
 #include "web-server.h"
 
+#include "requires.h"
 #include "test.h"
 
 int
@@ -57,17 +58,12 @@ main (int argc, char *argv[])
   char *data;
   CLEANUP_FREE char *usp_param = NULL;
 
-  if (access (disk, F_OK) == -1) {
-    fprintf (stderr, "%s: %s not found, test skipped\n", argv[0], disk);
-    exit (77);
-  }
-
 #ifndef HAVE_CURLOPT_UNIX_SOCKET_PATH
-  fprintf (stderr, "%s: curl does not support CURLOPT_UNIX_SOCKET_PATH\n",
-           argv[0]);
-  exit (77);
+  skip_because ("curl does not support CURLOPT_UNIX_SOCKET_PATH");
 #endif
 
+  requires_exists (disk);
+
   sockpath = web_server (disk, NULL, false);
   if (sockpath == NULL) {
     fprintf (stderr, "test-tar-xz-curl: could not start web server thread\n");
