@@ -42,6 +42,7 @@
 
 #include <guestfs.h>
 
+#include "requires.h"
 #include "test.h"
 
 int
@@ -51,6 +52,9 @@ main (int argc, char *argv[])
   int r;
   char *label;
 
+  /* Check we have an mkfs command. */
+  requires ("mkfs --version");
+
   /* Start nbdkit. */
   if (test_start_nbdkit ("tmpdisk", "1G", "label=TEST", NULL) == -1)
     exit (EXIT_FAILURE);
